@@ -207,7 +207,7 @@ class ImmutableRecordWithoutPickling(RecordWithoutPickling):
                     for field in self.__class__.fields))
 
 
-class ImmutableRecord(Record, ImmutableRecordWithoutPickling):
+class ImmutableRecord(ImmutableRecordWithoutPickling, Record):
     pass
 
 # }}}
@@ -1750,6 +1750,18 @@ def generate_unique_names(prefix):
         try_num += 1
 
 
+def generate_numbered_unique_names(prefix, num=None):
+    orig_num = num
+    num = 0
+    if orig_num is None:
+        yield (num, prefix)
+
+    while True:
+        name = "%s_%d" % (prefix, num)
+        num += 1
+        yield (num, name)
+
+
 generate_unique_possibilities = MovedFunctionDeprecationWrapper(
         generate_unique_names)
 
@@ -1758,10 +1770,21 @@ class UniqueNameGenerator:
     def __init__(self, existing_names=set(), forced_prefix=""):
         self.existing_names = existing_names.copy()
         self.forced_prefix = forced_prefix
+        self.prefix_to_counter = {}
 
     def is_name_conflicting(self, name):
         return name in self.existing_names
 
+    def _name_added(self, name):
+        """Callback to alert subclasses when a name has been added.
+
+        .. note::
+
+            This will not get called for the names in the *existing_names*
+            argument to :meth:`__init__`.
+        """
+        pass
+
     def add_name(self, name):
         if self.is_name_conflicting(name):
             raise ValueError("name '%s' conflicts with existing names")
@@ -1769,6 +1792,7 @@ class UniqueNameGenerator:
             raise ValueError("name '%s' does not start with required prefix")
 
         self.existing_names.add(name)
+        self._name_added(name)
 
     def add_names(self, names):
         for name in names:
@@ -1777,13 +1801,18 @@ class UniqueNameGenerator:
     def __call__(self, based_on="id"):
         based_on = self.forced_prefix + based_on
 
-        for var_name in generate_unique_names(based_on):
+        counter = self.prefix_to_counter.get(based_on, None)
+
+        for counter, var_name in generate_numbered_unique_names(based_on, counter):
             if not self.is_name_conflicting(var_name):
                 break
 
+        self.prefix_to_counter[based_on] = counter
+
         var_name = intern(var_name)
 
         self.existing_names.add(var_name)
+        self._name_added(var_name)
         return var_name
 
 # }}}
